#! /usr/bin/python3
# -*- coding: utf-8 -*-

# Copyright(C) 2013 name <email>
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

from gi.repository import GLib, Gio
from gi.repository import Unity
import gettext
import json
import urllib.request
import urllib.parse

APP_NAME = 'unity-scope-songsterr'
LOCAL_PATH = '/usr/share/locale/'
gettext.bindtextdomain(APP_NAME, LOCAL_PATH)
gettext.textdomain(APP_NAME)
_ = gettext.gettext

GROUP_NAME = 'com.canonical.Unity.Scope.Music.Songsterr'
UNIQUE_PATH = '/com/canonical/unity/scope/music/songsterr'

SEARCH_HINT = _('Search Songsterr')
NO_RESULTS_HINT = _('Sorry, there are no Songsterr results that match your search.')
PROVIDER_CREDITS = _('Powered by Songsterr')
SVG_DIR = '/usr/share/icons/unity-icon-theme/places/svg/'
PROVIDER_ICON = SVG_DIR + 'service-songsterr.svg'
DEFAULT_RESULT_ICON = SVG_DIR + 'service-songsterr.svg'
DEFAULT_RESULT_MIMETYPE = 'text/html'
DEFAULT_RESULT_TYPE = Unity.ResultType.DEFAULT

SEARCH_URI = 'http://www.songsterr.com/a/ra/songs.json?pattern=%s'

c1 = {'id': 'songs',
      'name': _('Tabs'),
      'icon': SVG_DIR + 'group-installed.svg',
      'renderer': Unity.CategoryRenderer.VERTICAL_TILE}
CATEGORIES = [c1]

FILTERS = []

m1 = {'id'   :'artist',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
m2 = {'id'   :'tabs',
      'type' :'s',
      'field':Unity.SchemaFieldType.OPTIONAL}
EXTRA_METADATA = [m1, m2]


def search(search, filters):
    '''
    Query the songsterr API
    '''
    results = []
    if not search or len(search) < 3:
        return results
    search = urllib.parse.quote(search)
    uri = SEARCH_URI % search
    print(uri)
    try:
        response = urllib.request.urlopen(uri).read()
        result = json.loads(response.decode('utf8'))
    except:
        result = {}
    if result:
        for item in result:
            if len(results) > 24:
                break
            results.append({'uri': 'http://www.songsterr.com/a/wa/song?id=%s' % item['id'],
                            'title': item['title'],
                            'artist': item['artist']['name'],
                            'tabs': ";".join(item['tabTypes'])})
    return results


class Preview (Unity.ResultPreviewer):

    def do_run(self):
        icon = Gio.FileIcon.new(Gio.file_new_for_uri(self.result.icon_hint))
        preview = Unity.GenericPreview.new(self.result.title, '', icon)
        preview.props.subtitle = self.result.metadata['artist'].get_string()
        tabs = self.result.metadata['tabs'].get_string().split(';')
        tab_types = {"PLAYER":None,"TEXT_GUITAR_TAB":_("Guitar"),"CHORDS":_("Chords"),"TEXT_BASS_TAB":_("Bass")}
        tab_list = []
        for t in tabs:
            if tab_types[t]:
                tab_list.append(tab_types[t])
        if len(tab_list) > 0:
            preview.add_info(Unity.InfoHint.new("tabs", _("Tabs"), None, ', '.join(tab_list)))
        icon = Gio.FileIcon.new (Gio.file_new_for_path(PROVIDER_ICON))
        view_action = Unity.PreviewAction.new("open", _("View"), icon)
        preview.add_action(view_action)
        return preview


# Classes below this point establish communication
# with Unity, you probably shouldn't modify them.


class MySearch(Unity.ScopeSearchBase):
    def __init__(self, search_context):
        super(MySearch, self).__init__()
        self.set_search_context(search_context)

    def do_run(self):
        '''
        Adds results to the model
        '''
        try:
            result_set = self.search_context.result_set
            for i in search(self.search_context.search_query,
                            self.search_context.filter_state):
                if not 'uri' in i or not i['uri'] or i['uri'] == '':
                    continue
                if not 'icon' in i or not i['icon'] or i['icon'] == '':
                    i['icon'] = DEFAULT_RESULT_ICON
                if not 'mimetype' in i or not i['mimetype'] or i['mimetype'] == '':
                    i['mimetype'] = DEFAULT_RESULT_MIMETYPE
                if not 'result_type' in i or not i['result_type'] or i['result_type'] == '':
                    i['result_type'] = DEFAULT_RESULT_TYPE
                if not 'category' in i or not i['category'] or i['category'] == '':
                    i['category'] = 0
                if not 'title' in i or not i['title']:
                    i['title'] = ''
                if not 'comment' in i or not i['comment']:
                    i['comment'] = ''
                if not 'dnd_uri' in i or not i['dnd_uri'] or i['dnd_uri'] == '':
                    i['dnd_uri'] = i['uri']
                result_set.add_result(**i)
        except Exception as error:
            print (error)


class Scope(Unity.AbstractScope):
    def __init__(self):
        Unity.AbstractScope.__init__(self)

    def do_get_search_hint(self):
        return SEARCH_HINT

    def do_get_schema(self):
        '''
        Adds specific metadata fields
        '''
        schema = Unity.Schema.new()
        if EXTRA_METADATA:
            for m in EXTRA_METADATA:
                schema.add_field(m['id'], m['type'], m['field'])
        #FIXME should be REQUIRED for credits
        schema.add_field('provider_credits', 's', Unity.SchemaFieldType.OPTIONAL)
        return schema

    def do_get_categories(self):
        '''
        Adds categories
        '''
        cs = Unity.CategorySet.new()
        if CATEGORIES:
            for c in CATEGORIES:
                cat = Unity.Category.new(c['id'], c['name'],
                                         Gio.ThemedIcon.new(c['icon']),
                                         c['renderer'])
                cs.add(cat)
        return cs

    def do_get_filters(self):
        '''
        Adds filters
        '''
        fs = Unity.FilterSet.new()
        #if FILTERS:
        #
        return fs

    def do_get_group_name(self):
        return GROUP_NAME

    def do_get_unique_name(self):
        return UNIQUE_PATH

    def do_create_search_for_query(self, search_context):
        se = MySearch(search_context)
        return se

    def do_create_previewer(self, result, metadata):
        rp = Preview()
        rp.set_scope_result(result)
        rp.set_search_metadata(metadata)
        return rp

def load_scope():
    return Scope()
